<?
/* Name: functions.php
 * Project: Remote Sensor based on AVR USB driver
 * Author: Christian Starkjohann
 * Creation Date: 2005-04-13
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 * This Revision: $Id: functions.php 314 2007-03-29 17:17:09Z cs $
 */

/* configuration: */

    $workdir = ".";
    $csvfile = "$workdir/data.csv";
    $avg30m = "$workdir/average-30m.csv";
    $avg2h = "$workdir/average-2h.csv";
    $avg1d = "$workdir/average-1d.csv";
    $avg7d = "$workdir/average-7d.csv";

/*
General Description:
This file contains utility functions used in the php scripts and configuration
constants.
*/
/* ------------------------------------------------------------------------- */

/* Returns Unix timestamp */
function parseTimestamp($string)
{
    if(!ereg("([0-9]{4})-([0-9]{1,2})-([0-9]{1,2}) ([0-2][0-9]):([0-5][0-9]):([0-5][0-9])", $string, $regs))    /* date in ISO */
        return 0;
    return mktime($regs[4], $regs[5], $regs[6], $regs[2], $regs[3], $regs[1]);
}

/* Returns Unix time */
function parseHumanInputTimestamp($string)
{
    if(ereg("([0-9]{2,4})-([0-9]{1,2})-([0-9]{1,2})( .*)?", $string, $regs)){   /* date in ISO */
        $year = $regs[1];
        $month = $regs[2];
        $day = $regs[3];
    }elseif(ereg("([0-9]{1,2}).([0-9]{1,2})-([0-9]{2,4})( .*)?", $string, $regs)){  /* euro format */
        $year = $regs[3];
        $month = $regs[2];
        $day = $regs[1];
    }elseif(ereg("([0-9]{1,2}).([0-9]{1,2})( .*)?", $string, $regs)){   /* short euro format */
        $month = $regs[2];
        $day = $regs[1];
        $x = getdate(time());
        $year = $x["year"];
    }else{
        return time();  /* no valid date format */
    }
    if($year < 100){
        if($year < 70){
            $year += 2000;
        }else{
            $year += 1900;
        }
    }
    return mktime(0, 0, 0, $month, $day, $year) + 86400;
}


function lastTimeInFile($filename)
{
    $offset = @filesize($filename) - 4096;
    if($offset < 0)
        $offset = 0;
    $fp = @fopen($filename, "r");
    if(!$fp)
        return 0;
    $t = 0;
    fseek($fp, $offset);
    if($offset != 0)
        fgets($fp, 4096);   /* read first incomplete line */
    while(($a = fgetcsv($fp, 256, ";")) !== FALSE){
        $x = parseTimestamp($a[0]);
        $t = $x != 0 ? $x : $a[0];
    }
    fclose($fp);
    return $t;
}

function openAtTime($filename, $time)   /* open and position at >= $time */
{
    $lower = 0;
    $upper = @filesize($filename);
    $fp = fopen($filename, "r");
    if(!$fp)
        return $fp;
    $lastOffset = -1;
    $offset = 0;
    for(;;){    /* do a binary search for the first date equal or greater $time */
        fseek($fp, $offset);
        if($offset != 0){
            fgets($fp, 4096);
            $offset = ftell($fp);   /* position after next nl */
        }
        if(($a = fgetcsv($fp, 256, ";")) !== FALSE){
            $x = parseTimestamp($a[0]);
            $t = $x != 0 ? $x : $a[0];
            if($t < $time){
                $lower = $offset;
            }else{
                $upper = $offset;
            }
        }else{  /* This must be EOF */
            $upper = $offset;
        }
        if($offset == $lastOffset)
            break;
        $lastOffset = $offset;
        $offset = ($upper + $lower - 8) / 2;
        if($offset < 0)
            $offset = 0;
    }
    fseek($fp, $upper);
    return $fp;
}

// $hue = [0..360], $saturation = [0..1], $brightness = [0..1]
function hsv2rgb($h, $s, $v)
{
    $v *= 255;
    if($s == 0){
        return array($v, $v, $v);
    }else{
        $hue = ($h % 360) / 60;
        $i = intval($hue);
        $f = $hue - $i;
        $w = $v * (1.0 - $s);
        $q = $v * (1.0 - ($s * $f));
        $t = $v * (1.0 - ($s * (1.0 - $f)));
        switch($i){
            case 0: return array($v, $t, $w);
            case 1: return array($q, $v, $w);
            case 2: return array($w, $v, $t);
            case 3: return array($w, $q, $v);
            case 4: return array($t, $w, $v);
            case 5: return array($v, $w, $q);
        }
    }
}

function buildColorTable()
{
global  $gColorTable;

    $hueInc = 720;
    $h = 0;
    for($i=0;$i<32;$i++){
        $s = $i & 1 ? 0.5 : 1;
        $v = $i & 1 ? 0.8 : 1;
        $gColorTable[$i] = hsv2rgb($h, $s, $v);
        if($i & 1){
            $h += $hueInc;
            if($h >= 360){
                $hueInc /= 2;
                $h = $hueInc / 2;
            }
        }
    }
}

function    colorForSensor($sensor, $isHumidity)
{
global  $gColorTable;

    if(!isset($gColorTable)){
        buildColorTable();
    }
    $i = $sensor * 2;
    if($isHumidity){
        $i ++;
    }
    return $gColorTable[$i];
}

function voltages()
{
global  $csvfile;

    $offset = @filesize($csvfile) - 8192;
    if($offset < 0)
        $offset = 0;
    $fp = @fopen($csvfile, "r");
    if(!$fp)
        return array();
    fseek($fp, $offset);
    if($offset != 0)
        fgets($fp, 4096);   /* read first incomplete line */
    while(($a = fgetcsv($fp, 512, ";")) !== FALSE){
        $v[(int)$a[1]] += $a[2];
        $cnt[(int)$a[1]]++;
    }
    fclose($fp);
    while(list($sensor, $c) = each($cnt)){
        $v[$sensor] /= $c;
    }
    return $v;
}

?>